///////////////////////////////////////////////////////////////////////////////
// File: frequency_counter_4MHz_LCD_TMR1.c
// Curtesy: J F Main.
//
// Description:
//
//   Measure frequency using TMR1 up to about 50MHz (depends on PIC chip input).
//   Displays result on an LCD (4 bit mode).
//
// Compiler : mikroC, mikroElektronika C compiler
//            for Microchip PIC microcontrollers
//            Version: 8.2.0.0
//
// Note Testing:
//            Tested on 16F877A
//
// Requirements:
//   Xtal clock : 4MHz
//     (Use any type but better (smaller) ppm specification = better accuracy).
//     (for different xtal frequency - gate_loop & tune).
//
//   Target : 16F877A
//
//   Accuracy: uses ONLY the main crystal clock as a reference
//         - so the counter accuracy depends on the crystal.
//
//   Note : re-tune gate_loop when ever code is added/removed.
//
//   Check: By using simulator timer see code in gate_loop
///////////////////////////////////////////////////////////////////////////////

#include "bit.h"
#include "delay.h"
#include "bcd.h"

// If the following are local the compiler optimses them out.
// Declare as global so not optimised by new compiler:
unsigned long  ldmy=0; // force compiler to keep these
unsigned short sdmy=0; // force compiler to keep these

//////////////////////////// POTRTS INITIALIZATION ////////////////////////////

void init_ports(void) {
   ADCON1 = 0x06; // all digital outputs
   PORTD = 0;
   TRISD = 0;     // out
   TRISC = 0x81;  // B7 = i/p for USART, RC0 T1CKI-Timer1 i/p.
}

////////////////////////////////// GATE LOOP //////////////////////////////////
// Action:
//    Reads the overflow flag in constant time
//    and causes a measurement time just short of 1 second.
//    Returns the overflow count from TIMER1
//
// GATE and REFRESH
//
// TUNE THIS GATE TIME to 1 second approx
//
//  1. Set the inner loop time to 999us
//     Break on statement "gt--"
//     Reset stopwatch
//     step until reach the original statement "gt--"
//     Adjust inner loop contents until have a loop time of 999us.
//
unsigned long gate_loop(unsigned int gt) org 200 {
unsigned long  oc = 0;

      do {
         gt--;

         // test for overflow
         sdmy = PIR1 & (1<<TMR1IF);
         asm {
               CLRW ;  // needed to lket simulator update var 5.0.0.3
           }

         if (sdmy!=0) {
            PIR1      &= ~(1<<TMR1IF);  // clear the bit.
            oc += 1;          // increase high count.
         } else {
           sdmy  &= ~(1<<TMR1IF);
           ldmy += 1;
           asm {
               CLRW ;
           }
         }
         delay_500_us();
         delay_100_us();
         delay_100_us();
         delay_100_us();
         delay_100_us();
         delay_10_us();
         delay_10_us();
         delay_10_us();
         delay_10_us();
         delay_10_us();
         asm {
            CLRW
            CLRW
            CLRW
         }
      } while (gt>0);

      return oc;
}

//////////////////////////////// MAIN FUNCTION ////////////////////////////////

void main() org 400 {
char op[12]                   ; // Display string for ascii converted long.
unsigned short st_TMR1L       ; // Store Timer 1 low byte (after calced).
unsigned short st_TMR1H       ; // store Timer 1 high byte time.
unsigned long  st_TMR1_ovfl=0 ; // store timer 1 overflows - multiples of 65536.
unsigned long  df = 0         ; // display frequency value to use.
unsigned short blinkc=0       ; // blink counter
char lcdop[4];
      
   init_ports();
   // Timer 1
   // prescale = 0, oscillator off, timer on, external clock, unsynchronized.
   T1CON = (1<<TMR1ON) | (1<<TMR1CS) | (1<<T1SYNC);
   // Enable  Timer 1 interrupt flag so that the flag is updated
   // interrupt not used just its output.
   PIE1 = (1<<TMR1IE) ; // TMR1 overflow flag is enabled and can be polled.
   // Note: actual interrupts not enabled.

   Lcd_Config(&PORTD,1,0,2,7,6,5,4);
   LCD_Cmd(LCD_CLEAR);
   LCD_Cmd(LCD_CURSOR_OFF);
   LCD_Out(1,4,"WELCOME TO");
   LCD_Out(2,3,"FREQ. COUNTER");
   delay_ms(2000);
   LCD_Cmd(LCD_CLEAR);

   // blink the led on port C.
   for(;;) {
      if (blinkc>0) { // processor alive indicator
         PORTC=0xff;
      } else {
         PORTC=0x00;
      }
      blinkc=~blinkc;

      // Calculate frequency from timer
      if (!(st_TMR1L==0 && st_TMR1H==0 && st_TMR1_ovfl==0) ) {
         df = st_TMR1L+(st_TMR1H<<8)+(st_TMR1_ovfl<<16);
      } else {
         df = 0;
      }

      ulong2bcd(df);
      process_ulong2str(op);
      
      LCD_Out(1,1,"f(input\)");
      LCD_Cmd(LCD_SECOND_ROW);
      Lcd_Cmd(LCD_RETURN_HOME);
      LCD_Out(2,1,op);

      // Initialize ready for next gate time
      st_TMR1L     = 0;
      st_TMR1H     = 0;
      st_TMR1_ovfl = 0;

      TMR1H = 0; // clear timer1 high count. stop overflow
      PIR1  &= ~(1<<TMR1IF);  // clear overflow bit.
      TMR1L = 0; // clear timer1 low count.

      // Note tune the gate time to exactly 1second
      //   1. Break on  st_TMR1_ovfl = gate_loop();
      //   2. Reset stopwatch
      //   3. step until reach statement "st_TMR1L = TMR1L;"
      //   4. Adjust statements between start and finish to give 1second.

      // Start of gate time
      st_TMR1_ovfl = gate_loop(1000);

      delay_500_us();
      delay_100_us();
      delay_100_us();
      delay_10_us();
      asm {
         CLRW
         CLRW
         CLRW
         CLRW
      }

      // get counter values before it updates
      st_TMR1L = TMR1L;
      st_TMR1H = TMR1H;

      LCD_Out_Cp(" Hz");
      // End of gate time
      
      // test for overflow after tune delay
      if ( PIR1 & (1<<TMR1IF) ) {
         PIR1      &= ~(1<<TMR1IF);  // clear the bit.
         st_TMR1_ovfl += 1;          // increase high count.
      }
   }
}






